@@ -6,9 +6,9 @@
 ### hhh4 is an extended version of algo.hhh for the sts-class
 ### The function allows the incorporation of random effects and covariates.
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2015 Sebastian Meyer
-### $Revision: 1475 $
-### $Date: 2015-09-14 22:38:55 +0200 (Mon, 14. Sep 2015) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2016 Sebastian Meyer
+### $Revision: 1706 $
+### $Date: 2016-05-03 16:09:49 +0200 (Die, 03. Mai 2016) $
 ################################################################################
 
 ## Error message issued in loglik, score and fisher functions upon NA parameters
@@ -89,10 +89,24 @@ hhh4 <- function (stsObj, control = list(
                     cntrl.variance   = control$optimizer$variance,
                     verbose=control$verbose)
 
+  ## extract parameter estimates
   convergence <- myoptim$convergence == 0
-  thetahat <- c(myoptim$fixef, myoptim$ranef)
-  loglik <- myoptim$loglik
+  thetahat <- myoptim$theta
+  if (dimRandomEffects>0) {
+      Sigma.orig <- myoptim$sd.corr
+      Sigma.trans <- getSigmai(head(Sigma.orig,model$nVar),
+                               tail(Sigma.orig,model$nCorr),
+                               model$nVar)
+      dimnames(Sigma.trans) <-
+          rep.int(list(sub("^sd\\.", "",
+                           names(Sigma.orig)[seq_len(model$nVar)])), 2L)
+  } else {
+      Sigma.orig <- Sigma.trans <- NULL
+  }
+
+  ## compute covariance matrices of regression and variance parameters
   cov <- try(solve(myoptim$fisher), silent=TRUE)
+  Sigma.cov <- if(dimRandomEffects>0) try(solve(myoptim$fisherVar), silent=TRUE)
 
   ## check for degenerate fisher info
   if(inherits(cov, "try-error")){ # fisher info is singular
@@ -106,44 +120,23 @@ hhh4 <- function (stsObj, control = list(
   }
   if (!convergence) {
       if (control$verbose) {
-          cat("Penalized loglikelihood =", loglik, "\n")
+          cat("Penalized loglikelihood =", myoptim$loglik, "\n")
           thetastring <- paste(round(thetahat,2), collapse=", ")
           thetastring <- strwrap(thetastring, exdent=10, prefix="\n", initial="")
           cat("theta = (", thetastring, ")\n")
       }
       warning("Results are not reliable!", ADVICEONERROR)
-      res <- myoptim
-      res$convergence <- convergence
-      res$call <- match.call()
-      class(res) <- "hhh4"
-      return(res)
   }
 
-  ## optimization successful, return a full "hhh4" object
-  dimnames(cov) <- list(names(thetahat), names(thetahat))
-  if (dimRandomEffects>0) {
-      Sigma.orig <- myoptim$sd.corr
-      Sigma.cov <- solve(myoptim$fisherVar)
-      dimnames(Sigma.cov) <- list(names(Sigma.orig),names(Sigma.orig))
-      Sigma.trans <- getSigmai(head(Sigma.orig,model$nVar),
-                               tail(Sigma.orig,model$nCorr),
-                               model$nVar)
-      dimnames(Sigma.trans) <-
-          rep.int(list(sub("^sd\\.", "",
-                           names(Sigma.orig)[seq_len(model$nVar)])), 2L)
-  } else {
-      Sigma.orig <- Sigma.cov <- Sigma.trans <- NULL
-  }
-  
-  ## Done
-  result <- list(coefficients=thetahat, se=sqrt(diag(cov)), cov=cov, 
+  ## gather results in a list -> "hhh4" object
+  result <- list(coefficients=thetahat,
+                 se=if (convergence) sqrt(diag(cov)), cov=cov, 
                  Sigma=Sigma.trans,     # estimated covariance matrix of ri's
                  Sigma.orig=Sigma.orig, # variance parameters on original scale
                  Sigma.cov=Sigma.cov,   # covariance matrix of Sigma.orig
                  call=match.call(),
                  dim=c(fixed=dimFixedEffects,random=dimRandomEffects),
-                 loglikelihood=loglik,
-                 margll=marLogLik(Sigma.orig, thetahat, model),
+                 loglikelihood=myoptim$loglik, margll=myoptim$margll,
                  convergence=convergence,
                  fitted.values=meanHHH(thetahat, model, total.only=TRUE),
                  control=control,
@@ -155,8 +148,12 @@ hhh4 <- function (stsObj, control = list(
                  nTime=length(model$subset), nUnit=ncol(stsObj),
                  ## CAVE: nTime is not nrow(stsObj) as usual!
                  runtime=proc.time()-ptm)
+  if (!convergence) {
+      ## add (singular) Fisher information for further investigation
+      result[c("fisher","fisherVar")] <- myoptim[c("fisher","fisherVar")]
+  }
   class(result) <- "hhh4"
-  result
+  return(result)
 }
 
 
@@ -961,9 +958,8 @@ penLogLik <- function(theta, sd.corr, model, attributes=FALSE)
   lpen <- if (dimRE==0) 0 else { # there are random effects
     ##-.5*(t(randomEffects)%*%Sigma.inv%*%randomEffects)
     ## the following implementation takes ~85% less computing time !
-    -0.5 * crossprod(randomEffects, Sigma.inv) %*% randomEffects
+    -0.5 * c(crossprod(randomEffects, Sigma.inv) %*% randomEffects)
   }
-  lpen <- c(lpen)                       # drop 1x1 matrix dimensions
   
   ## log-likelihood
   ll.units <- .colSums(model$family(Y,mu,psi),
@@ -1528,8 +1524,8 @@ marLogLik <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
   # where -0.5*log(|Sigma|) = -dim(RE_i)*[Sum(sd_i) -0.5*log(1+corr_i^2)]
   ##lpen <- -0.5*(t(randomEffects)%*%Sigma.inv%*%randomEffects)
   ## the following implementation takes ~85% less computing time !
-  lpen <- -0.5 * crossprod(randomEffects, Sigma.inv) %*% randomEffects
-  loglik.pen <- sum(-dimBlocks*sd) + c(lpen)
+  lpen <- -0.5 * c(crossprod(randomEffects, Sigma.inv) %*% randomEffects)
+  loglik.pen <- sum(-dimBlocks*sd) + lpen
   if(dimCorr >0){
     loglik.pen <- loglik.pen + 0.5*dimBlocks[1]*sum(log(1+corr^2))
   }
@@ -1609,11 +1605,11 @@ marScore <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
     dS.i <- getSigma(dimSigma=dimVar,dimBlocks=dimBlocks,Sigmai=dSi)
     #dlpen.i <- -0.5* t(randomEffects) %*% dS.i %*% randomEffects
     # ~85% faster implementation using crossprod() avoiding "slow" t():
-    dlpen.i <- -0.5 * crossprod(randomEffects, dS.i) %*% randomEffects
+    dlpen.i <- -0.5 * c(crossprod(randomEffects, dS.i) %*% randomEffects)
     #tr.d1logDetF <- sum(diag(F.inv.RE %*% dS.i))
     tr.d1logDetF <- sum(F.inv.RE * dS.i)   # since dS.i is symmetric
     #<- needs 1/100 (!) of the computation time of sum(diag(F.inv.RE %*% dS.i))
-    marg.score[i] <- d1logDet[i] + c(dlpen.i) - 0.5 * tr.d1logDetF
+    marg.score[i] <- d1logDet[i] + dlpen.i - 0.5 * tr.d1logDetF
   }
   
   return(marg.score)
@@ -1719,7 +1715,7 @@ marFisher <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
       #d2lpen.i <- -0.5* t(randomEffects) %*% dSij %*% randomEffects
       # ~85% faster implementation using crossprod() avoiding "slow" t():
-      d2lpen.i <- -0.5 * crossprod(randomEffects, dSij) %*% randomEffects
+      d2lpen.i <- -0.5 * c(crossprod(randomEffects, dSij) %*% randomEffects)
 
       # compute second derivative of log-determinant of penFisher
       mpart1 <- dS.j %*% F.inv.RE  # 3 times as fast as the other way round
@@ -2139,11 +2135,15 @@ fitHHH <- function(theta, sd.corr, model,
   
   ll <- penLogLik(theta=theta,sd.corr=sd.corr,model=model)
   fisher <- penFisher(theta=theta,sd.corr=sd.corr,model=model)
+  dimnames(fisher) <- list(names(theta), names(theta))
+  margll <- marLogLik(sd.corr=sd.corr, theta=theta, model=model)
   fisher.var <- marFisher(sd.corr=sd.corr, theta=theta, model=model,
                           fisher.unpen=fisher.unpen)
+  dimnames(fisher.var) <- list(names(sd.corr), names(sd.corr))
   
-  list(fixef=head(theta,dimFE.d.O), ranef=tail(theta,dimRE), sd.corr=sd.corr,
-       loglik=ll, fisher=fisher, fisherVar=fisher.var,
+  list(theta=theta, sd.corr=sd.corr,
+       loglik=ll, margll=margll,
+       fisher=fisher, fisherVar=fisher.var,
        convergence=convergence, dim=c(fixed=dimFE.d.O,random=dimRE))
 }
 
